/**********************************************************************************************************************
 * DISCLAIMER
 * This software is supplied by Renesas Electronics Corporation and is only intended for use with Renesas products. No
 * other uses are authorized. This software is owned by Renesas Electronics Corporation and is protected under all
 * applicable laws, including copyright laws.
 * THIS SOFTWARE IS PROVIDED "AS IS" AND RENESAS MAKES NO WARRANTIES REGARDING
 * THIS SOFTWARE, WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING BUT NOT LIMITED TO WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NON-INFRINGEMENT. ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED. TO THE MAXIMUM
 * EXTENT PERMITTED NOT PROHIBITED BY LAW, NEITHER RENESAS ELECTRONICS CORPORATION NOR ANY OF ITS AFFILIATED COMPANIES
 * SHALL BE LIABLE FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES FOR ANY REASON RELATED TO
 * THIS SOFTWARE, EVEN IF RENESAS OR ITS AFFILIATES HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
 * Renesas reserves the right, without notice, to make changes to this software and to discontinue the availability of
 * this software. By using this software, you agree to the additional terms and conditions found by accessing the
 * following link:
 * http://www.renesas.com/disclaimer
 *
 * Copyright (C) 2020 Renesas Electronics Corporation. All rights reserved.
 *********************************************************************************************************************/
/***********************************************************************/
/*                                                                     */
/*  FILE        :Main.c or Main.cpp                                    */
/*  DATE        :Tue, Oct 31, 2006                                     */
/*  DESCRIPTION :Main Program                                          */
/*  CPU TYPE    :                                                      */
/*                                                                     */
/*  NOTE:THIS IS A TYPICAL EXAMPLE.                                    */
/*                                                                     */
/***********************************************************************/
//#include "typedefine.h"
#include <machine.h>
#include "r_cg_macrodriver.h"
#include "iodefine.h"
#include "Config_ICU.h"
#include "Config_PORT.h"
#include "Config_SCI5.h"
#include "Config_TMR1.h"
#include "Config_TPU2.h"
#include "Config_CMT0.h"
#include "Config_S12AD0.h"
#include "r_cg_userdefine.h"
#include "sci.h"
#include "lcd_1602.h"
#include "io_board.h"

#ifdef __cplusplus
//#include <ios>                        // Remove the comment when you use ios
//_SINT ios_base::Init::init_cnt;       // Remove the comment when you use ios
#endif

void main(void);
#ifdef __cplusplus
extern "C" {
void abort(void);
}
#endif

void main(void)
{

	unsigned char xc;
	unsigned short ret;
	unsigned char buzz_cycle;
	unsigned char buzz_cycle_tmp;
	unsigned long pwm_tmp;
	unsigned short adc_disp_val = 0;
	short adc_disp_diff;
	unsigned char prev_matrix_sw_no = 0;
	unsigned char matrix_sw_no = 0;
	unsigned char prev_matrix_sw_pushed_count = 0;
	unsigned char matrix_sw_pushed_count = 0;
	unsigned short stepping_motor_pos = 0;
	unsigned short stepping_motor_exit = 0;
	unsigned short i;
	
	sci_start();
	
	R_Config_CMT0_Start();
	
	R_Config_ICU_IRQ4_Start();
	R_Config_ICU_IRQ13_Start();
	R_Config_ICU_IRQ15_Start();
	
	g_adc_flag = 1;
	R_Config_S12AD0_Start();
	
	//BUZZ, LED (P17,P15) -> Hi-Z
	PORT1.PMR.BIT.B5 = 0;
	PORT1.PMR.BIT.B7 = 0;  
	PORT1.PDR.BIT.B5 = 0;
	PORT1.PDR.BIT.B7 = 0;
	
	sci_write_str("\n");
	
	sci_write_str("HSBRX65-IO-BOARD[since Ver1.1](HSBRX651F144A) Sample program [Rev1.3].\n");
	sci_write_str("Copyright (C) 2019-2022 HokutoDenshi. All Rights Reserved.\n");
	
	sci_write_str("TEST MENU:\n");
	sci_write_str("  b : buzzer(P17) [ON/OFF toggle]\n");
	sci_write_str("  l : LED8(P15) [ON/OFF toggle]\n");
	sci_write_str("  m : key matrix\n");
	sci_write_str("  s : stepping motor[CCW]\n");
	sci_write_str("  r : stepping motor[CW]\n");
	sci_write_str("  7 : 7seg Demo\n");
	sci_write_str("  h : help\n");
	
	sci_write_str(">");
	
	lcd_init();									//LCD̏
	
	lcd_clear();
	lcd_hs1();
	//             1234567890123456
	lcd_write_str("HSBRX65-IO-BOARD");
	lcd_hs2();
	
	//TMR1TCN
	buzz_pitch(g_buzz_cycle_ar[0]);	//ቹ
	
   	while(1)
	{
		ret = sci_read_char(&xc);
		if(ret != SCI_RECEIVE_DATA_EMPTY)
		{
			switch(xc)
			{
				case 'b':
					sci_write_char(xc);
					sci_write_str("\n buzzer (P17)>\n");
				
					if(g_buzz_flag == ON)
					{
						g_buzz_flag=OFF;
						sci_write_str(" ->buzzer OFF\n");
					
						R_Config_TMR1_Stop();
					
						//BUZZ (P17) -> Hi-Z
						PORT1.PMR.BIT.B7 = 0;  
						PORT1.PDR.BIT.B7 = 0;
					
						//TPM1TCN
						buzz_pitch(g_buzz_cycle_ar[0]);	//ቹ
					}
					else
					{
						g_buzz_flag=ON;
						sci_write_str(" ->buzzer ON\n");
					
						//BUZZ (P17) -> timer
						PORT1.PMR.BIT.B7 = 1;  
					
						R_Config_TMR1_Start();
					}
					sci_write_str("\n>");

					break;
	
				case 'l':
					sci_write_char(xc);
					sci_write_str("\n LED8 (P15)>\n");
				
					if(g_led8_flag == ON)
					{
						g_led8_flag=OFF;
						sci_write_str(" ->LED8 OFF\n");
					
						R_Config_TPU2_Stop();
					
						//LED8 (P15) -> Hi-Z
						PORT1.PMR.BIT.B5 = 0;
						PORT1.PDR.BIT.B5 = 0;
					}
					else
					{
						g_led8_flag=ON;
						sci_write_str(" ->LED8 blink ON\n");
					
						//LED8 (P15) -> timer
						PORT1.PMR.BIT.B5 = 1; 
					
						R_Config_TPU2_Start();
					}
					sci_write_str("\n>");

					break;
			
				case 'm':
					sci_write_char(xc);
					sci_write_str("\n -matrix sw (press any key to exit)-\n");
					sci_write_str("SW11 SW12 SW13 SW14 SW15 SW16 SW17 SW18 SW19 SW20 SW21 SW22 SW23 SW24 SW25 SW26\n");
					while(1)
					{
						for(i=0; i<4; i++)
						{
							if(g_matrix_sw[i] & 0x0001) sci_write_str("  -  ");
							else  sci_write_str("  o  ");
						
							if(g_matrix_sw[i] & 0x0002) sci_write_str("  -  ");
							else  sci_write_str("  o  ");
						
							if(g_matrix_sw[i] & 0x0004) sci_write_str("  -  ");
							else  sci_write_str("  o  ");
						
							if(g_matrix_sw[i] & 0x0008) sci_write_str("  -  ");
							else  sci_write_str("  o  ");
	
						}
						sci_write_char(0x0d);
						mdelay(10);
					
						ret = sci_read_char(&xc);
						if(ret != SCI_RECEIVE_DATA_EMPTY) break;
					}
					sci_write_str("\n>");

					break;
				
				case 's':
					sci_write_char(xc);
					sci_write_str("\n -stepping motor drive [CCW] (press any key to exit)-\n");
					stepping_motor_en(1);
					stepping_motor_exit=0;
					while(1)
					{
						for(i=0; i<4; i++)
						{
							stepping_motor_pos = (unsigned char)i;
							stepping_motor_drive(stepping_motor_pos);
							mdelay(250);
					
							ret = sci_read_char(&xc);
							if(ret != SCI_RECEIVE_DATA_EMPTY)
							{
								stepping_motor_exit=1;
								break;
							}
						}
						if(stepping_motor_exit == 1) break;
					}
				
					stepping_motor_en(0);
					sci_write_str("\n>");
				
					break;
		
				case 'r':
					sci_write_char(xc);
					sci_write_str("\n -stepping motor drive [CW] (press any key to exit)-\n");
					stepping_motor_en(1);
					stepping_motor_exit=0;
					while(1)
					{
						for(i=0; i<4; i++)
						{
							stepping_motor_pos = (unsigned char)(3-i);
							stepping_motor_drive(stepping_motor_pos);
							mdelay(250);
					
							ret = sci_read_char(&xc);
							if(ret != SCI_RECEIVE_DATA_EMPTY)
							{
								stepping_motor_exit=1;
								break;
							}
						}
						if(stepping_motor_exit == 1) break;
					}
				
					stepping_motor_en(0);
					sci_write_str("\n>");
				
					break;
				
				case '7':
					sci_write_char(xc);
					sci_write_str("\n 7seg LED\n");
					g_seg_state=ON;
					sci_write_str(" ->7seg led DEMO -> ON\n");
					
					seg_led(0x8888, 0x000f);
					mdelay(3000);
					seg_led(0x1111, 0x0001);
					mdelay(500);
					seg_led(0x2222, 0x0002);
					mdelay(500);
					seg_led(0x3333, 0x0003);
					mdelay(500);
					seg_led(0x4444, 0x0004);
					mdelay(500);
					seg_led(0x5555, 0x0005);
					mdelay(500);
					seg_led(0x6666, 0x0006);
					mdelay(500);
					seg_led(0x7777, 0x0007);
					mdelay(500);
					seg_led(0x8888, 0x0008);
					mdelay(500);
					seg_led(0x9999, 0x0009);
					mdelay(500);
					seg_led(0xaaaa, 0x000a);
					mdelay(500);
					seg_led(0xbbbb, 0x000b);
					mdelay(500);
					seg_led(0xcccc, 0x000c);
					mdelay(500);
					seg_led(0xdddd, 0x000d);
					mdelay(500);
					seg_led(0xeeee, 0x000e);
					mdelay(500);
					seg_led(0xffff, 0x000f);
					mdelay(500);
				
					sci_write_str("\n>");
				
					break;
				
					case 'h':
						sci_write_char(xc);
						sci_write_str("\n");
						sci_write_str("  b : buzzer(P17) [ON/OFF toggle]\n");
						sci_write_str("  l : LED8(P15) [ON/OFF toggle]\n");
						sci_write_str("  m : key matrix\n");
						sci_write_str("  s : stepping motor[CCW]\n");
						sci_write_str("  r : stepping motor[CW]\n");
						sci_write_str("  7 : 7seg Demo\n");
						sci_write_str("  h : help\n");
						sci_write_str("\n>");
					break;
				
				
				default:
					if(xc == '\r' || xc == '\n') sci_write_str("\n>");
					else if(ret != SCI_RECEIVE_DATA_EMPTY) sci_write_char(xc);
					break;
			}
		}
		
		//SW0-7, LED0-7
		sw_led();
		
		//uU[
		if(g_buzz_flag == ON)
		{
			if(PORTD.PIDR.BIT.B7 == 0) buzz_cycle_tmp=0;
			else if(PORTD.PIDR.BIT.B6 == 0) buzz_cycle_tmp=1;
			else if(PORTD.PIDR.BIT.B5 == 0) buzz_cycle_tmp=2;
			else if(PORTD.PIDR.BIT.B4 == 0) buzz_cycle_tmp=3;
			else if(PORTD.PIDR.BIT.B3 == 0) buzz_cycle_tmp=4;
			else if(PORTD.PIDR.BIT.B2 == 0) buzz_cycle_tmp=5;
			else if(PORTD.PIDR.BIT.B1 == 0) buzz_cycle_tmp=6;
			else if(PORTD.PIDR.BIT.B0 == 0) buzz_cycle_tmp=7;
			
			//ύXƂ̂݃^C}WX^XV
			if(buzz_cycle != buzz_cycle_tmp)
			{
				buzz_cycle = buzz_cycle_tmp;
				buzz_pitch(g_buzz_cycle_ar[buzz_cycle]);
			}
		}
		
		//R29-ADC
		if(g_adc_flag == 0)
		{
			g_adc_flag = 1;
			R_Config_S12AD0_Start();
		}
		
		//P15 PWM
		if(g_led8_flag == ON)
		{
			pwm_tmp = (unsigned long)TPU2.TGRA * (unsigned long)g_adc_val[0];
			g_tpu2_tgrb = (unsigned short)(pwm_tmp / 4096);						//TPU2.TGRBlO[oϐɕۑiXV̓JE^NAj
		}
		
		//7SEG LED
		switch(g_seg_state)
		{
			case ON:
				g_seg_flag=ON;
				if(g_adc_val_update == 1)
				{
					adc_disp_val=g_adc_val[0];
					g_adc_val_update=0;
				}
				else
				{
					adc_disp_diff = g_adc_val[0] - adc_disp_val;
					if(adc_disp_diff < 0) adc_disp_diff*=-1;
					if(adc_disp_diff > ADC_VAL_HIST) adc_disp_val=g_adc_val[0];
				}
				seg_led(adc_disp_val, 0);
				break;
				
			case OFF:
				g_seg_flag=OFF;
				break;
				
			case HOLD:
				g_seg_flag=ON;
				break;
				
			default:
				break;
		}
			
		
		//SW11-26
		matrix_sw_no=0;
		matrix_sw_pushed_count=0;
		
		for(i=0; i<4; i++)
		{
			if(!(g_matrix_sw[i] & 0x0001))
			{
				matrix_sw_no = 10 + 1 + i*4;
				matrix_sw_pushed_count++;
			}
			
			if(!(g_matrix_sw[i] & 0x0002))
			{
				matrix_sw_no = 10 + 2 + i*4;
				matrix_sw_pushed_count++;
			}
			
			if(!(g_matrix_sw[i] & 0x0004))
			{
				matrix_sw_no = 10 + 3 + i*4;
				matrix_sw_pushed_count++;
			}
			
			if(!(g_matrix_sw[i] & 0x0008))
			{
				matrix_sw_no = 10 + 4 + i*4;
				matrix_sw_pushed_count++;
			}	
		}
		
		if((prev_matrix_sw_no != matrix_sw_no))
		{
			lcd_hs2();
			if(matrix_sw_no == 0)
			{
				//             SW11
				lcd_write_str("     ");
			}
			else
			{
				lcd_write_str("SW");
				lcd_write_byte_int(matrix_sw_no);
				lcd_write_str(" ");
			}
		}
		
		if(matrix_sw_pushed_count > 1)
		{
			//2ȏL[Ăꍇ"SW23*"̗lɁA*\
			lcd_cmd(0xc0 + 4);		//2s5ڂw
			lcd_write_str("*");
		}
		else if((prev_matrix_sw_pushed_count > 1) && (matrix_sw_pushed_count <= 1))
		{
			//2ȏL[ĂԂꂽ
			lcd_cmd(0xc0 + 4);		//2s5ڂw
			lcd_write_str(" ");
		}
		
		prev_matrix_sw_no=matrix_sw_no;
		prev_matrix_sw_pushed_count=matrix_sw_pushed_count;
		
		nop(); nop(); nop(); nop(); nop(); nop(); nop(); nop();
	}
}

#ifdef __cplusplus
void abort(void)
{

}
#endif
